###
## SecClabRa
## Copyright (C) 2003 SecClab group (mail. lfp70@hotmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public Licence as published by
# the Free Software Foundation; eihter version 2 of the License, or
# (at your option) any later version.
#
# This program is distributes in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
##

from Globals import DTMLFile,InitializeClass,HTML
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from App.Common import package_home
from DateTime import DateTime
import os
import sys
import sha
import string
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from OFS.Image import Image,File
import time
import OpensslWrapper
import exceptions

from SecClabCa import MessageDialog

# WEB_CERTIFICATION_REQUEST_MAIL="""From:%s\r\nTo:%s\r\nSubject:%s\r\n\r\n
WEB_CERTIFICATION_REQUEST_MAIL="""

You have registered your certification request successfully.

Your registration id is %s.

You can verify the status of your request at:

%s
"""
RA_DENIED_MAIL = """

Your certification request has been denied because:

%s
"""
RA_CERTIFIED_MAIL = """

Your request has been certified.

You can download the certificate from:
%s?request=%s
"""

STATUS_PENDING   = 'pending'
STATUS_ERROR     = 'error'
STATUS_CERTIFIED = 'certified'
STATUS_DENIED    = 'denied'

STATUS_LIST = [STATUS_PENDING,
               STATUS_ERROR,
               STATUS_CERTIFIED,
               STATUS_DENIED]

STATUS_RA_PENDING  = 'pending'
STATUS_RA_APPROVED = 'approved'
STATUS_RA_DENIED   = 'denied'

STATUS_RA_LIST = [STATUS_RA_PENDING,
                  STATUS_RA_APPROVED,
                  STATUS_RA_DENIED]

CERT_COMMON_NAME_FIELD              = "commonName"
CERT_EMAIL_ADDRESS_FIELD            = "emailAddress"
CERT_ORGANIZATIONAL_UNIT_NAME_FIELD = "organizationalUnitName"
CERT_ORGANIZATION_NAME_FIELD        = "organizationName"
CERT_LOCALITY_NAME_FIELD            = "localityName"
CERT_STATE_OR_PROVICE_NAME_FIELD    = "stateOrProvinceName"
CERT_COUNTRY_NAME_FIELD             = "countryName"



manage_addSecClabRaForm = DTMLFile('dtml/manage_addSecClabRaForm',globals())
manage_addSecClabWebForm = DTMLFile('dtml/manage_addSecClabWebForm',globals())
def manage_addSecClabRa (self, id,
                         title='',
                         mailFrom='',
                         mailhost='127.0.0.1',
                         url_getcert='',
                         REQUEST=None):
    """ Add a new SecClabRa to current ObjectManager """

    ra = SecClabRa(id,title,mailFrom,url_getcert)
    self._setObject(id,ra)
    ra = self._getOb(id)

    ra._templateReset(package_home(globals()))

    ra.manage_addFolder(id='REQs')
    ra.manage_addFolder(id='CERTs')

    ra.manage_addProduct['MailHost'].manage_addMailHost('MailHost', 
                                                        smtp_host=mailhost)
    
    if REQUEST:
        return self.manage_main(self,REQUEST,update_menu=1)

def manage_addSecClabWeb (self, id,
                          title='',
                          mailhost='127.0.0.1',
                          mailFrom='root@127.0.0.1',
                          rafolder='',
                          REQUEST=None):
    """ Add a new SecClabWeb to current ObjectManager """    
    web = SecClabWeb(id,title,mailFrom,rafolder)
    self._setObject(id,web)
    web = self._getOb(id)

    web.manage_addProduct['MailHost'].manage_addMailHost('MailHost', 
                                                         smtp_host=mailhost)
    
    web._templateReset(package_home(globals()))
    
    if REQUEST:
        return self.manage_main(self,REQUEST,update_menu=1)




class SecClabRa(Folder):
    """ a folder with RA-managing-features  """

    security = ClassSecurityInfo()
    
    meta_type = 'SecClab Registration Authority'
    title = ''

    _properties=(
        {'id':'title',      'type':'string', 'mode':'w'},
        {'id':'mailFrom','type':'string','mode':'w'},
        {'id':'url_getcert','type':'string','mode':'w'},
        {'id':'crl','type':'lines','mode':'w'},
        {'id':'ca','type':'lines','mode':'w'},
        )
    def __init__(self,id,title,mailFrom,url_getcert):
        """ setup a SecClabRa """
        self.id = id
        self.title = title
        self.mailFrom = mailFrom
        self.url_getcert = url_getcert


    def _templateFile(self,directory,file,title='',type='PageTemplate'):
        f = open("%s/%s" %(directory,file),"rb");data=f.read();f.close()
        if type == 'PageTemplate':
            t = ZopePageTemplate(file,data,"text/html")
            t.pt_setTitle(title)
            self._setObject(file,t)
        elif type == 'Image':
            t = Image(file,title,data)
            self._setObject(file,t)
        elif type == 'File':
            t = File(file,title,data)
            self._setObject(file,t)
        else:
            self.manage_addDTMLMethod(file,title,data)
            
    def _templateCreation(self,directory):
        for file in os.listdir(directory):
            extension = string.split(file,'.')[-1]
            if extension == 'webprj':
                continue
            elif extension == 'gif':
                self._templateFile(directory,file,type='Image')
            elif extension == 'css':
                self._templateFile(directory,file,type='DTML')
            elif extension == 'dll':
                self._templateFile(directory,file,type='File')
            else:
                self._templateFile(directory,file)

    def _templateReset(self,prefix):
        self._templateCreation("%s/templates/ra/default" % prefix)


    def GetExtendedInfoRequest(self,id,not_html=0):
        info_request = self.GetInfoRequest(id,1)
        if info_request['type'] == 'spkac':
            info = OpensslWrapper.GetInfoFromSpkac(info_request['content'])
        else:
            info = OpensslWrapper.GetInfoFromPkcs10(info_request['content'])


        lines = string.split(info,'\n')
        for l in range(len(lines)-1):
            if (len(lines[l]) > 0):
                if lines[l][0] == ' ':
                    counter = 1
                    while counter < len(lines[l]):
                        if lines[l][counter] != ' ':
                            break
                        counter = counter + 1
                    for i in range(0,counter,2):
                        lines[l] =  '&nbsp;' + lines[l]

        info_request = self.GetInfoRequest(id,not_html)
        info_request['extended_content'] = string.join(lines,'<br>\n')


        return info_request
        
    def GetInfoRequest(self,id,not_html=0):
        req = getattr(self.REQs,id)

        if not_html == 0:
            MAXLINE = 70
            content_lines = string.split(str(req.request),'\n')
            result_content_lines = []
            for content_line in content_lines:
                chunk = content_line[0:MAXLINE]
                result_content_lines.append(chunk)
                for i in range(MAXLINE, len(content_line), MAXLINE - 3):
                    result_content_lines[-1] = result_content_lines[-1] + "..."
                    chunk = content_line[i : i + MAXLINE - 3]
                    result_content_lines.append("..."+chunk)
            string_content = string.join(result_content_lines,"\n")
        else:
            string_content = str(req.request)

        
        req_info = {'id':req.id,
                    'policy':req.policy,
                    #'date':req.bobobase_modification_time().strftime('%Y/%m/%d %H:%M:%S'),
                    'date':req.date_created,
                    'type':req.type,
                    'status':req.status,
                    'date_approved':req.date_approved,
                    'ra_status':req.ra_status,
                    'error':req.error,
                    'denial_reason':req.denial_reason,
                    'notification_sent':req.notification_sent,
                    'content':string_content
                    }

        return req_info

    def CreateCertObject4Req(self,id):
        req = getattr(self.REQs,id)
        req.manage_changeProperties({'status':STATUS_CERTIFIED})
        self.CERTs.manage_addFolder(id=id)

    def SaveCert4Req(self,id,cert):
        folder = getattr(self.CERTs,id)
        folder.manage_addProduct['OFSP'].manage_addFile(id='certificate', title='certificate',
                                                        file=cert,content_type='application/x-x509-user-cert')        

    def GetRequestListApproved(self,sort_by,status_req):
        return self._getRequestList(sort_by,status_req,[STATUS_RA_APPROVED])

    def GetRequestList(self,sort_by,status_req,status_ra):

        if len(status_ra) == 0:
            status_list = list(STATUS_RA_LIST)
        else:
            status_list = []
            if 'p' in status_ra:
                status_list.append(STATUS_RA_PENDING)
            if 'a' in status_ra:
                status_list.append(STATUS_RA_APPROVED)
            if 'd' in status_ra:
                status_list.append(STATUS_RA_DENIED)
                
        return self._getRequestList(sort_by,status_req,status_list)

    def _getRequestList(self,sort_by,status_req,status_ra_list):
        def SortRequestFunction(e1,e2):
            if (e1['sort_by'] > e2['sort_by']):
                return 1
            elif (e1['sort_by'] < e2['sort_by']):
                return -1
            else:
                return 0

        if len(status_req) == 0:
            status_list = list(STATUS_LIST)
        else:
            status_list = []
            if 'p' in status_req:
                status_list.append(STATUS_PENDING)
            if 'e' in status_req:
                status_list.append(STATUS_ERROR)
            if 'c' in status_req:
                status_list.append(STATUS_CERTIFIED)
            if 'd' in status_req:
                status_list.append(STATUS_DENIED)
        
        req_list = self.REQs.objectValues('Folder')
        r_list = []

        for req in req_list:
            if req.ra_status not in status_ra_list:
                continue
            if req.status in status_list:
                elem = {'id':req.id,
                        'policy':req.policy,
                        # 'date':req.bobobase_modification_time().strftime('%Y/%m/%d %H:%M:%S'),
                        'date':req.date_created,
                        'type':req.type,
                        'status':req.status,
                        'date_approved':req.date_approved,
                        'ra_status':req.ra_status,
                        'error':req.error,
                        'denial_reason':req.denial_reason,
                        'notification_sent':req.notification_sent}
                elem['sort_by'] = elem[sort_by]
                r_list.append(elem)
        r_list.sort(SortRequestFunction)
        return r_list
            

    def AddCertificateRequest(self,id,content,type,policy,private_info=None):
        request_id = "K_"+id
        self.REQs.manage_addFolder(id=request_id)
        folder = getattr(self.REQs,request_id)
        folder.manage_addProduct['OFSP'].manage_addFile(id='request', title='request',file=content,content_type='text/plain')
        folder.manage_addProperty(id='type',value=type,type='string')
        folder.manage_addProperty(id='status',value=STATUS_PENDING,type='string')
        folder.manage_addProperty(id='policy',value=policy,type='string')
        folder.manage_addProperty(id='date_created',value=DateTime(time.time()),type='date')
        folder.manage_addProperty(id='ra_status',value=STATUS_RA_PENDING,type='string')
        folder.manage_addProperty(id='date_approved',value=DateTime(time.time()),type='date')
        folder.manage_addProperty(id='error',value="",type='string')
        folder.manage_addProperty(id='denial_reason',value='',type='string')
        folder.manage_addProperty(id='notification_sent',value=0,type='boolean')
                                  
        if private_info is not None:
            folder.manage_addFolder('private_info')
            private_folder = getattr(folder,'private_info')
            for entry in private_info:
                if entry['type'] == 'file':
                    private_folder.manage_addProduct['OFSP'].manage_addFile(id=entry['id'], title=entry['title'],
                                                                            file=entry['content'],content_type=entry['content_type'])
                else:
                    private_folder.manage_addProperty(id=entry['id'],value=entry['content'],type='string')
        return request_id
    
    security.declarePublic('manage_modifyRequest')
    def manage_modifyRequest(self,REQUEST):
        """ manage_modifyRequest"""
        cgi_values = [['request',REQUEST.form['request']]]
        url_requests = string.split(REQUEST.form['url_requests'],'&')
        for var in url_requests:
            pair = string.split(var,'=')
            if len(pair) == 1:
                cgi_values.append([pair,''])
            else:
                cgi_values.append([pair[0],pair[1]])
        
        info_req = self.GetInfoRequest(REQUEST.form['request'])
        req = getattr(self.REQs,REQUEST.form['request'])
        if info_req['ra_status'] == STATUS_RA_APPROVED  or info_req['status']==STATUS_CERTIFIED or info_req['ra_status'] == STATUS_RA_DENIED:
            return MessageDialog(
                title  ='Error modifying a request',
                message='You can not modify a request that is approved or certificated',
                action ='edit_request',
                cgi_values=cgi_values
                )
        else:
            req.manage_changeProperties({'type':REQUEST.form['type'],
                                         'policy':REQUEST.form['policy']})
            req.request.update_data(REQUEST.form['content'])
            
            return MessageDialog(
                title  ='Modify success',
                message='This request has been modified',
                action ='edit_request',
                cgi_values=cgi_values
                )        



    security.declarePublic('manage_approveRequest')
    def manage_approveRequest(self,REQUEST):
        """ manage_approveRequest"""
        cgi_values = [['request',REQUEST.form['request']]]
        url_requests = string.split(REQUEST.form['url_requests'],'&')
        for var in url_requests:
            pair = string.split(var,'=')
            if len(pair) == 1:
                cgi_values.append([pair,''])
            else:
                cgi_values.append([pair[0],pair[1]])
        
        info_req = self.GetInfoRequest(REQUEST.form['request'])
        req = getattr(self.REQs,REQUEST.form['request'])
        if (info_req['ra_status'] == STATUS_RA_APPROVED and info_req['status'] != STATUS_ERROR) or info_req['status']==STATUS_CERTIFIED:
            return MessageDialog(
                title  ='Error approving a request',
                message='You can not approve a request that is approved or certificated',
                action ='edit_request',
                cgi_values=cgi_values
                )
        else:
            req.manage_changeProperties({'ra_status':STATUS_RA_APPROVED,'date_approved':DateTime(time.time()),
                                         'status':STATUS_PENDING})
            return MessageDialog(
                title  ='Approval success',
                message='This request has been approved',
                action ='edit_request',
                cgi_values=cgi_values
                )
        
    security.declarePublic('manage_denyRequest')
    def manage_denyRequest(self,REQUEST):
        """ manage_denyRequest"""
        cgi_values = [['request',REQUEST.form['request']]]
        url_requests = string.split(REQUEST.form['url_requests'],'&')
        for var in url_requests:
            pair = string.split(var,'=')
            if len(pair) == 1:
                cgi_values.append([pair,''])
            else:
                cgi_values.append([pair[0],pair[1]])
                
        info_req = self.GetInfoRequest(REQUEST.form['request'])
        req = getattr(self.REQs,REQUEST.form['request'])
        if info_req['ra_status'] == STATUS_RA_DENIED or info_req['status']==STATUS_CERTIFIED:
            return MessageDialog(
                title  ='Error deniyng a request',
                message='You can not deny a request that is denied or certificated',
                action ='edit_request',
                cgi_values=cgi_values
                )
        else:
            req.manage_changeProperties({'ra_status':STATUS_RA_DENIED,
                                         'denial_reason':REQUEST.form['reason']})        
            return MessageDialog(
                title  ='Denial success',
                message='This request has been denied',
                action ='edit_request',
                cgi_values=cgi_values
                )
    security.declarePublic('manage_pendRequest')
    def manage_pendRequest(self,REQUEST):
        """ manage_pendRequest"""
        cgi_values = [['request',REQUEST.form['request']]]
        url_requests = string.split(REQUEST.form['url_requests'],'&')
        for var in url_requests:
            pair = string.split(var,'=')
            if len(pair) == 1:
                cgi_values.append([pair,''])
            else:
                cgi_values.append([pair[0],pair[1]])
                
        info_req = self.GetInfoRequest(REQUEST.form['request'])
        req = getattr(self.REQs,REQUEST.form['request'])
        if info_req['ra_status'] == STATUS_RA_PENDING or info_req['status'] == STATUS_CERTIFIED:
            return MessageDialog(
                title  ='Error marking pending a request',
                message='You can not mark pending a request that is pending or certified',
                action ='edit_request',
                cgi_values=cgi_values
                )
        else:
            req.manage_changeProperties({'ra_status':STATUS_RA_PENDING})        
            return MessageDialog(
                title  ='Mark pending',
                message='This request has been marked pending',
                action ='edit_request',
                cgi_values=cgi_values
                )
    security.declarePublic("manage_sendNotifications")
    def manage_sendNotifications(self):
        """ manage_sendNotifications"""
        request_list = self.GetRequestList('id',['c'],[]) + self.GetRequestList('id',[],['d'])
        notifications_sent = 0
        notifications_error = 0
        error_en_envio = ""
        for request in request_list:
            if request['notification_sent']:
                continue
            req_object = getattr(self.REQs,request['id'])
            email_request = req_object.private_info.email
            if request['ra_status'] == STATUS_RA_DENIED:
                message = RA_DENIED_MAIL % request['denial_reason']
            elif request['status'] == STATUS_CERTIFIED:
                message = RA_CERTIFIED_MAIL % (self.url_getcert,request['id'])
                
            try:
                req_object.manage_changeProperties({'notification_sent':1})
                self.MailHost.send(message, mto=email_request, mfrom=self.mailFrom, subject="Response to certification request from Registration Authority")
                
                get_transaction().commit()
                notifications_sent = notifications_sent + 1
            except:
                get_transaction().abort()
                notifications_error = notifications_error + 1
                if error_en_envio == "":
                    error_en_envio = "%s<br>%s<br>%s" % (sys.exc_type,sys.exc_value,sys.exc_traceback)
                
        return MessageDialog(
            title="Notifications",
            message="Notifications succesfully sent: %d, <br>Notifications not sent:%d<br>%s" % (notifications_sent,
                                                                                                 notifications_error,
                                                                                                 error_en_envio),
            action="notifications")

    def SetRequestError(self,request_id,error):
        req_object = getattr(self.REQs,request_id)
        req_object.manage_changeProperties({'status':STATUS_ERROR,
                                            'error':error})

    def SetRequestDenied(self,request_id,reason):
        req_object = getattr(self.REQs,request_id)
        req_object.manage_changeProperties({'status':STATUS_DENIED,
                                            'error':reason})
        
    security.declarePublic('manage_requestAction')    
    def manage_requestAction(self,REQUEST):
        """ manage_requestAction """
        return MessageDialog(
            title = 'Method not implemented',
            message = 'METHOD NOT IMPLENTED',
            action='javascript:history.go(-1);')

    def PublishCRLandCAcerts(self,crl_pem,p7_ca_pem):
        self.manage_changeProperties({'crl':crl_pem,
                                      'ca':p7_ca_pem});

    
class SecClabWeb(Folder):
    """ a folder with Web-conector """

    security = ClassSecurityInfo()
    
    meta_type = 'SecClab Web Connector'
    title = ''
    rafolder = ''

    _properties=(
        {'id':'title',      'type':'string', 'mode':'w'},
        {'id':'rafolder',        'type':'string', 'mode':'w'},
        {'id':'mailFrom','type':'string','mode':'w'},
        {'id':'policies','type':'lines','mode':'w'},
        )
    def __init__(self,id,title,mailFrom,rafolder):
        """ setup a Webconnector """
        self.id = id
        self.title = title
        self.rafolder = rafolder
        self.mailFrom = mailFrom
        self.policies = ""

    def SetRaFolder(self,rafolder):
        self.rafolder = rafolder

    def _templateFile(self,file,title='',prefix='',type='PageTemplate'):
        f = open("%s/templates/web/default/%s" % (prefix,file),"rb");data=f.read();f.close()
        if type == 'PageTemplate':
            t = ZopePageTemplate(file,data,"text/html")
            t.pt_setTitle(title)
            self._setObject(file,t)
        elif type == 'Image':
            t = Image(file,title,data)
            self._setObject(file,t)
        elif type == 'File':
            t = File(file,title,data)
            self._setObject(file,t)
        else:
            self.manage_addDTMLMethod(file,title,data)
    def _templateReset(self,prefix):
        for file in os.listdir("%s/templates/web/default" % prefix):
            extension = string.split(file,'.')[-1]
            if extension == 'webprj':
                continue
            elif extension == 'gif':
                self._templateFile(file,file,prefix,'Image')
            elif extension == 'css':
                self._templateFile(file,file,prefix,'DTML')
            elif extension == 'dll':
                self._templateFile(file,file,prefix,'File')
            else:
                self._templateFile(file,file,prefix)

    def GetRaFolder(self):
        if len(self.rafolder) == 0:
            raise 'ERROR', 'RA folder value not valid. Please contact with administrator site'
        try:
            raobject = self.restrictedTraverse(string.split(self.rafolder,'/'))
        except exceptions.KeyError:
            raise 'ERROR','RA folder not valid. Please contact with administrator site'
        except:
            raise 'ERROR','Please contact with administrator site (%s-%s-%s)' % (sys.exc_type,sys.exc_value,sys.exc_traceback)

        if raobject.meta_type != 'SecClab Registration Authority':
            raise 'ERROR','RA folder is not an RA Object. Please contact with administrator site'
        return raobject

    def GetInfoRequest(self,request_id):
        if request_id == "":
            return None
        try:
            ra_object = self.GetRaFolder()
            req_info = ra_object.GetInfoRequest(request_id)
        except exceptions.AttributeError:
            return None
        except:
            raise #return None
        return req_info

    def GetInfoCertificate(self,request_id):
        try:
            ra_object = self.GetRaFolder()
            cert_object = getattr(ra_object.CERTs,request_id,None)
            if cert_object is None:
                return None

            info = OpensslWrapper.GetCertificateInfo(str(cert_object.certificate))

            lines = string.split(info,'\n')
            for l in range(len(lines)-1):
                if (len(lines[l]) > 0):
                    if lines[l][0] == ' ':
                        counter = 1
                        while counter < len(lines[l]):
                            if lines[l][counter] != ' ':
                                break
                            counter = counter + 1
                        for i in range(0,counter,2):
                            lines[l] =  '&nbsp;' + lines[l]
            return string.join(lines,'<br>\n')            
        except:
            return None
        

    security.declarePublic("manage_addCARequest")
    def manage_addCARequest(self,REQUEST):
        """ manage_addCARequest """
        return self.manage_addServerRequest(REQUEST)
    
    security.declarePublic("manage_addServerRequest")
    def manage_addServerRequest(self,REQUEST):
        """ manage_addServerRequest """
        try:
            ra_object = self.GetRaFolder()
        except:
            return MessageDialog(
                title   = sys.exc_type,
                message = sys.exc_value,
                action = 'index_html'
                )

        h = sha.new()
        h.update(REQUEST.form['request'])
        id = h.hexdigest()

        private_info = [{'id':'email','type':'string','content':REQUEST.form['emailAddress']}]
        if REQUEST.form.has_key('policy'):
            policy = REQUEST.form['policy']
        else:
            policy = ''
        request_id = ra_object.AddCertificateRequest(id,REQUEST.form['request'],'pkcs10',policy,private_info)
        try:
            message = WEB_CERTIFICATION_REQUEST_MAIL %(id,REQUEST.URL1+'/get_cert?request='+request_id)
            self.MailHost.send(message,mto=REQUEST.form['emailAddress'],mfrom=self.mailFrom,subject="Certification request on WEB connector %s" % self.id)
        except:
            return MessageDialog(
                title  ='Request registered but error in email...',
                message='Certification request registered with id %s, but email error' % request_id,
                action ='index_html')
        
        return MessageDialog(
            title  ='Request registered...',
            message='Certification request registered with id %s' % request_id,
            action ='index_html')
    
    security.declarePublic('manage_addCertificateRequest')
    def manage_addCertificateRequest(self,REQUEST):
        """ manage_addCertificateRequest"""
        try:
            ra_object = self.GetRaFolder()
        except:
            return MessageDialog(
                title   = sys.exc_type,
                message = sys.exc_value,
                action = 'index_html'
                )

        if REQUEST.form.has_key('spkac'):

            h = sha.new()
            h.update(REQUEST.form['spkac'])
            id = h.hexdigest()

            spkac_text = "SPKAC=%s\n" % string.replace(string.replace(REQUEST.form['spkac'],'\r\n',''),'\n','')

            if REQUEST.form.has_key(CERT_COMMON_NAME_FIELD):
                spkac_text = spkac_text + "CN=%s\n" % REQUEST.form[CERT_COMMON_NAME_FIELD]
            if REQUEST.form.has_key(CERT_EMAIL_ADDRESS_FIELD):
                spkac_text = spkac_text + "emailAddress=%s\n" % REQUEST.form[CERT_EMAIL_ADDRESS_FIELD]
            if REQUEST.form.has_key(CERT_ORGANIZATIONAL_UNIT_NAME_FIELD):
                spkac_text = spkac_text + "OU=%s\n" % REQUEST.form[CERT_ORGANIZATIONAL_UNIT_NAME_FIELD]
            if REQUEST.form.has_key(CERT_ORGANIZATION_NAME_FIELD):
                spkac_text = spkac_text + "O=%s\n" % REQUEST.form[CERT_ORGANIZATION_NAME_FIELD]
            if REQUEST.form.has_key(CERT_LOCALITY_NAME_FIELD):
                spkac_text = spkac_text + "L=%s\n" % REQUEST.form[CERT_LOCALITY_NAME_FIELD]
            if REQUEST.form.has_key(CERT_STATE_OR_PROVICE_NAME_FIELD):
                spkac_text = spkac_text + "ST=%s\n" % REQUEST.form[CERT_STATE_OR_PROVICE_NAME_FIELD]
            if REQUEST.form.has_key(CERT_COUNTRY_NAME_FIELD):
                spkac_text = spkac_text + "C=%s\n" % REQUEST.form[CERT_COUNTRY_NAME_FIELD]
                                                   
            private_info = [{'id':'email','type':'string','content':REQUEST['emailAddress']}]
            if REQUEST.form.has_key('policy'):
                policy = REQUEST.form['policy']
            else:
                policy = ''
            request_id = ra_object.AddCertificateRequest(id,spkac_text,'spkac',policy,private_info)
        elif REQUEST.form.has_key('pkcs10'):
            h = sha.new()
            h.update(REQUEST.form['pkcs10'])
            id = h.hexdigest()

            p10req = "-----BEGIN CERTIFICATE REQUEST-----\n" + REQUEST.form['pkcs10'] + "-----END CERTIFICATE REQUEST-----"

            private_info = [{'id':'email','type':'string','content':REQUEST['emailAddress']}]
            if REQUEST.form.has_key('policy'):
                policy = REQUEST.form['policy']
            else:
                policy = ''

            request_id = ra_object.AddCertificateRequest(id,p10req,'pkcs10',policy,private_info)
        else:
            return MessageDialog(
                title  ='Error in form...',
                message='Certification request NOT registered',
                action ='index_html')


        try:
            message = WEB_CERTIFICATION_REQUEST_MAIL % (request_id,REQUEST.URL1+'/get_cert?request='+request_id)
            #(self.mailFrom,
            #                                            REQUEST.form['emailAddress'],
             #                                           'Certification request on WEB connector %s' % self.id,
             #                                           request_id,
             #                                           REQUEST.URL1+'/view_status_request?id='+request_id)
            
            self.MailHost.send(message,mto=REQUEST.form['emailAddress'],mfrom=self.mailFrom,subject="Certification request on WEB connector %s" % self.id)
            # smtpserver = smtplib.SMTP(self.MailHost.smtp_host, 
            #                           int(self.MailHost.smtp_port))
            # smtpserver.sendmail(self.mailFrom, REQUEST.form['emailAddress'],message)
            # smtpserver.quit()
        except:
            return MessageDialog(
                title  ='Request registered but error in email...',
                message='Certification request registered with id %s, but email error' % request_id,
                action ='index_html')
        
        return MessageDialog(
            title  ='Request registered...',
            message='Certification request registered with id %s' % request_id,
            action ='index_html')

    security.declarePublic('manage_installCertificate')
    def manage_installCertificate(self,REQUEST):
        """ manage_installCertificate """
        return self._download_certificate(REQUEST,REQUEST.response)
        
        
    security.declarePublic('manage_downloadCertificate')
    def manage_downloadCertificate(self,REQUEST):
        """ manage_downloadCertificate """
        return self._download_certificate(REQUEST,REQUEST.response,content_type='application/unknown-application')

    def _download_certificate(self,REQUEST,RESPONSE,content_type=None):
        try:
            ra_object = self.GetRaFolder()
        except:
            return MessageDialog(
                title   = sys.exc_type,
                message = sys.exc_value,
                action = 'javascript:history.go(-1);'
                )
        cert_object = getattr(ra_object.CERTs,REQUEST.form['request'],None)
        if cert_object is None:
            return MessageDialog(
                title = "Certificate for this id does not exist",
                message = "Certificate for this id does not exist",
                action = "javascript:history.go(-1);")

        bincert = str(cert_object.certificate)
        if content_type is None:
            content_type =cert_object.certificate.getContentType()
            RESPONSE.setHeader('Content-Type', content_type+";name=%s.cer" % (REQUEST.form['request']))
            RESPONSE.setHeader('Content-Length', len(bincert))
            RESPONSE.setHeader('Content-Disposition','inline; filename=%s.cer' % (REQUEST.form['request']))
        else:
            RESPONSE.setHeader('Content-Type', content_type+";name=%s.cer" % (REQUEST.form['request']))
            RESPONSE.setHeader('Content-Length', len(bincert))
            RESPONSE.setHeader('Content-Disposition','inline; filename=%s.cer' % (REQUEST.form['request']))

        RESPONSE.write(bincert)


    security.declarePublic('manage_downloadCACert')
    def manage_downloadCACert(self,REQUEST,RESPONSE):
        """download ca cert """
        try:
            ra_object = self.GetRaFolder()

            ca_cert_pem = string.join(ra_object['ca'],"\n")

            if len(ca_cert_pem) == 0:
                return MessageDialog(
                    title = "CA certificate does not exist",
                    message = "CA certificate does not exist",
                    action = "javascript:history.go(-1);")

            bincert = OpensslWrapper.P7SwitchOutput(ca_cert_pem,from_type='pem')
        except:
            return MessageDialog(
                title   = sys.exc_type,
                message = sys.exc_value,
                action = 'javascript:history.go(-1);'
                )

        # RESPONSE.setHeader('Content-Type', 'application/x-pkcs7-certificates;name=ca.spc')
        RESPONSE.setHeader('Content-Type', 'application/x-x509-ca-cert;name=ca.spc')
        RESPONSE.setHeader('Content-Length', len(bincert))
        RESPONSE.setHeader('Content-Disposition','inline; filename=ca.spc')
        RESPONSE.write(bincert)
        
    security.declarePublic('manage_downloadCRL')
    def manage_downloadCRL(self,REQUEST,RESPONSE):
        """download crl """
        try:
            ra_object = self.GetRaFolder()

            crl_pem = string.join(ra_object['crl'],"\n")
            
            if len(crl_pem) == 0:
                return MessageDialog(
                    title = "CRL does not exist",
                    message = "CRL certificate does not exist",
                    action = "javascript:history.go(-1);")

            bincrl = OpensslWrapper.CrlSwitchOutput(crl_pem,from_type='pem')
        except:
            return MessageDialog(
                title   = sys.exc_type,
                message = sys.exc_value,
                action = 'javascript:history.go(-1);'
                )

        RESPONSE.setHeader('Content-Type', 'application/x-x509-crl;name=crl.crl')
        RESPONSE.setHeader('Content-Length', len(bincrl))
        RESPONSE.setHeader('Content-Disposition','inline; filename=crl.crl')
        RESPONSE.write(bincrl)
        
InitializeClass(SecClabRa)
InitializeClass(SecClabWeb)
