###
## SecClabWebClabAPI
## Copyright (C) 2005 SecClab group (mail. lfp70@hotmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public Licence as published by
# the Free Software Foundation; eihter version 2 of the License, or
# (at your option) any later version.
#
# This program is distributes in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
##

from Globals import DTMLFile,InitializeClass,HTML,HTMLFile,MessageDialog
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from OFS.Image import File
from App.Common import package_home
from DateTime import DateTime
from SecClabApiResponse import SecClabApiResponse
from ZODB.POSException import ConflictError

import OFS.SimpleItem 
import Acquisition                # Various acquisition types. 
import AccessControl.Role         # Zope Security management. 
import exceptions

import os
import sys
import sha
import string
import traceback
import OpensslWrapper
import SecClabRa

manage_addSecClabWebClabAPIForm = DTMLFile('dtml/manage_addSecClabWebClabAPIForm',globals())

def manage_addSecClabWebClabAPI (self, id,
                                 title='',
                                 rafolder='',
                                 policy='',
                                 REQUEST=None):
    """ Add a new SecClabWebClabAPI to current ObjectManager """    
    web = SecClabWebClabAPI(id,title,rafolder,policy)
    self._setObject(id,web)
    web = self._getOb(id)

    objectNew = SecClabApiResponse("xmlLogin.jsp",title="xmlLogin",function_map={'':'Echo'})
    web._setObject("xmlLogin.jsp",objectNew)
    objectNew = SecClabApiResponse("apiProtocol.jsp",title="apiProtocol",function_map={'SendRequestCertificate':'SendRequestCertificate',
                                                                                       'GetCertificate':'GetCertificate'})
    web._setObject("apiProtocol.jsp",objectNew)

    web._not_deleted_objects = ["xmlLogin.jsp","apiProtocol.jsp"]
    
    if REQUEST:
        return self.manage_main(self,REQUEST,update_menu=1)



class SecClabWebClabAPI(Folder):
    """ a folder with CLAB API interface """

    security = ClassSecurityInfo()
    
    meta_type = 'SecClab Web CLAB API Connector'
    title = ''
    rafolder = ''
    policy = ''

    _properties=(
        {'id':'title',      'type':'string', 'mode':'w'},
        {'id':'rafolder',        'type':'string', 'mode':'w'},
        {'id':'policy',        'type':'string', 'mode':'w'},
        )
    def __init__(self,id,title,rafolder,policy):
        """ setup a Webconnector """
        self.id = id
        self.title = title
        self.rafolder = rafolder
        self.policy = policy
        self._not_deleted_objects = []

    def _getText(self,nodelist,encoding="iso-8859-15"):
         rc = ""
         for node in nodelist:
             if node.nodeType == node.TEXT_NODE:
                 rc = rc + node.data.encode(encoding)
         return rc

    def SetRaFolder(self,rafolder):
        self.rafolder = rafolder

    def GetRaFolder(self):
        if len(self.rafolder) == 0:
            raise 'ERROR', 'RA folder value not valid. Please contact with administrator site'
        try:
            raobject = self.restrictedTraverse(string.split(self.rafolder,'/'))
        except exceptions.KeyError:
            raise 'ERROR','RA folder not valid. Please contact with administrator site'
        except:
            raise 'ERROR','Please contact with administrator site (%s-%s-%s)' % (sys.exc_type,sys.exc_value,sys.exc_traceback)

        if raobject.meta_type != 'SecClab Registration Authority':
            raise 'ERROR','RA folder is not an RA Object. Please contact with administrator site'
        return raobject

    def _delObject(self, id, dp=1):
        if id in self._not_deleted_objects:
            raise ConflictError
        Folder._delObject(self,id,dp)
    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
        if id in self._not_deleted_objects:
            raise ConflictError
        return Folder._setObject(self,id,object,roles,user,set_owner)

    def ConstructResponseError(self,decodedXml,error,ext_error,RESPONSE):
        # tell zope undo all operations
        get_transaction().abort()
        responseNode = decodedXml.getElementsByTagName("response")[-1]
        
        node = decodedXml.createElement("ExitCode")
        node.appendChild(decodedXml.createTextNode("1"))
        responseNode.appendChild(node)

        node = decodedXml.createElement("ErrorMsg")
        node.appendChild(decodedXml.createTextNode(error.encode('utf8')))
        responseNode.appendChild(node)

        node = decodedXml.createElement("ExtendedErrorMsg")
        node.appendChild(decodedXml.createTextNode(ext_error.encode('utf8')))
        responseNode.appendChild(node)
        #decodedXml.normalize()

        message = decodedXml.toxml().encode('utf-8')
        RESPONSE.setHeader("content-type",'text/xml; charset="utf-8"')
        RESPONSE.setHeader('Content-Length', len(message))
        RESPONSE.write(message)
        return
    
        

    def Echo(self,decodedXml,REQUEST,RESPONSE):
        RESPONSE.setHeader("content-type",'text/xml; charset="utf-8"')
        if hasattr(REQUEST,'BODY'):
            RESPONSE.setHeader('Content-Length', len(REQUEST['BODY']))
            RESPONSE.write(REQUEST['BODY'])
        else:
            RESPONSE.setHeader('Content-Length', len('<?xml version="1.0"?><root/>'))
            RESPONSE.write('<?xml version="1.0"?><root/>')

    def SendRequestCertificate(self,decodedXml,REQUEST,RESPONSE):
        responseNode = decodedXml.getElementsByTagName("response")[-1]
        idNode = decodedXml.createElement("ID")
        responseNode.appendChild(idNode)

        try:
            ra_object = self.GetRaFolder()
        except:
            error = "%s" % sys.exc_info()[1]
            ext_error = "%s\r\n%s\r\n%s" % (sys.exc_info()[0],sys.exc_info()[1],string.join(traceback.format_list(traceback.extract_tb(sys.exc_info()[2])),"\r\n"))
            self.ConstructResponseError(decodedXml,error,ext_error,RESPONSE)
            return

        tipoRequest = self._getText(decodedXml.getElementsByTagName("TREQUEST")[0].childNodes)

        if tipoRequest == 'SPKAC':
            spkac = self._getText(decodedXml.getElementsByTagName("REQUEST")[0].childNodes)

            h = sha.new()
            h.update(spkac)
            id = h.hexdigest()

            spkac_text = "SPKAC=%s\n" % string.replace(string.replace(spkac,'\r\n',''),'\n','')
            email=""

            nodes = decodedXml.getElementsByTagName("CN")
            if len(nodes)>0:
                spkac_text = spkac_text + "CN=%s\n" % self._getText(nodes[0].childNodes)
            nodes = decodedXml.getElementsByTagName("EMAIL")
            if len(nodes)>0:
                email = self._getText(nodes[0].childNodes)
                spkac_text = spkac_text + "emailAddress=%s\n" % email
            nodes = decodedXml.getElementsByTagName("OU")
            if len(nodes)>0:
                spkac_text = spkac_text + "OU=%s\n" % self._getText(nodes[0].childNodes)
            nodes = decodedXml.getElementsByTagName("O")
            if len(nodes)>0:
                spkac_text = spkac_text + "O=%s\n" % self._getText(nodes[0].childNodes)
            nodes = decodedXml.getElementsByTagName("L")
            if len(nodes)>0:
                spkac_text = spkac_text + "L=%s\n" % self._getText(nodes[0].childNodes)
            nodes = decodedXml.getElementsByTagName("ST")
            if len(nodes)>0:
                spkac_text = spkac_text + "ST=%s\n" % self._getText(nodes[0].childNodes)
            nodes = decodedXml.getElementsByTagName("C")
            if len(nodes)>0:
                spkac_text = spkac_text + "C=%s\n" % self._getText(nodes[0].childNodes)
                                                   
            private_info = [{'id':'email','type':'string','content':email}]
            
            policy = self.policy
            
            request_id = ra_object.AddCertificateRequest(id,spkac_text,'spkac',policy,private_info)
        elif tipoRequest == 'PKCS10':
            pkcs10 = self._getText(decodedXml.getElementsByTagName("REQUEST")[0].childNodes)

            pkcs10 = '\n'.join([pkcs10[i : i+64] for i in range(0, len(pkcs10), 64)])
            h = sha.new()
            h.update(pkcs10)
            id = h.hexdigest()

            nodes = decodedXml.getElementsByTagName("EMAIL")
            email = ""
            if len(nodes)>0:
                email = self._getText(nodes[0].childNodes)

            p10req = "-----BEGIN CERTIFICATE REQUEST-----\n" + pkcs10 + "\n-----END CERTIFICATE REQUEST-----"

            private_info = [{'id':'email','type':'string','content':email}]
            policy = self.policy

            request_id = ra_object.AddCertificateRequest(id,p10req,'pkcs10',policy,private_info)
        else:
            self.ConstructResponseError(decodedXml,"Certification request NOT registered (%s)" % tipoRequest,"",RESPONSE)
            return
        # return request_id
        idNode.appendChild(decodedXml.createTextNode(request_id))
        
        node = decodedXml.createElement("ExitCode")
        node.appendChild(decodedXml.createTextNode("0"))
        responseNode.appendChild(node)

        node = decodedXml.createElement("ErrorMsg")
        node.appendChild(decodedXml.createTextNode(""))
        responseNode.appendChild(node)

        node = decodedXml.createElement("ExtendedErrorMsg")
        node.appendChild(decodedXml.createTextNode(""))
        responseNode.appendChild(node)

        decodedXml.normalize()

        message = decodedXml.toxml().encode('utf-8')
        RESPONSE.setHeader("content-type",'text/xml; charset="utf-8"')
        RESPONSE.setHeader('Content-Length', len(message))
        RESPONSE.write(message)
        return        

    def GetCertificate(self,decodedXml,REQUEST,RESPONSE):
        responseNode = decodedXml.getElementsByTagName("response")[-1]
        certificateNode = decodedXml.createElement("Certificate")
        responseNode.appendChild(certificateNode)
        
        try:
            ra_object = self.GetRaFolder()
        except:
            error = "%s" % sys.exc_info()[1]
            ext_error = "%s\r\n%s\r\n%s" % (sys.exc_info()[0],sys.exc_info()[1],string.join(traceback.format_list(traceback.extract_tb(sys.exc_info()[2])),"\r\n"))
            self.ConstructResponseError(decodedXml,error,ext_error,RESPONSE)
            return

        req_id = self._getText(decodedXml.getElementsByTagName("ID")[0].childNodes)
        
        cert_object = getattr(ra_object.CERTs,req_id,None)
        if cert_object is None:
            req_object = getattr(ra_object.REQs,req_id,None)
            if req_object is None:
                self.ConstructResponseError(decodedXml,"Certificate for this id does not exist","",RESPONSE)
                return
            else:
                if req_object['status'] != SecClabRa.STATUS_PENDING:
                    if req_object['status'] == SecClabRa.STATUS_ERROR:
                        error = "DENIED REASON: %s" % req_object['error']
                    elif req_object['status'] == SecClabRa.STATUS_DENIED:
                        error = "DENIED REASON: %s" % req_object['denial_reason']
                    else:
                        error = "UNDEFINED ERROR"

                    self.ConstructResponseError(decodedXml,error,"",RESPONSE)
                    return
                else:
                    if req_object['ra_status'] == SecClabRa.STATUS_RA_DENIED:
                        error = "DENIED REASON: %s" % req_object['denial_reason']
                        self.ConstructResponseError(decodedXml,error,"",RESPONSE)
                        return
            pemcert = ""
        else:
            bincert = str(cert_object.certificate)
            pemcert = OpensslWrapper.Certder2Certpem(bincert)
            
        # return certificate
        responseNode = decodedXml.getElementsByTagName("response")[-1]

        certificateNode.appendChild(decodedXml.createTextNode(pemcert))
        
        node = decodedXml.createElement("ExitCode")
        node.appendChild(decodedXml.createTextNode("0"))
        responseNode.appendChild(node)

        node = decodedXml.createElement("ErrorMsg")
        node.appendChild(decodedXml.createTextNode(""))
        responseNode.appendChild(node)

        node = decodedXml.createElement("ExtendedErrorMsg")
        node.appendChild(decodedXml.createTextNode(""))
        responseNode.appendChild(node)
        decodedXml.normalize()

        message = decodedXml.toxml().encode('utf-8')
        RESPONSE.setHeader("content-type",'text/xml; charset="utf-8"')
        RESPONSE.setHeader('Content-Length', len(message))
        RESPONSE.write(message)
        return                

        


InitializeClass(SecClabWebClabAPI)
